
///////////////////////////////////////////////////////////////////////////////////////////////
// power-framework: Java library with many purposes of usage.
// Copyright (C) 2016-2022 the original author or authors.
//
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation;
// version 2.1 of the License only.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
///////////////////////////////////////////////////////////////////////////////////////////////

package org.nanoboot.powerframework.random.generators;

import org.nanoboot.powerframework.core.exceptions.UnsupportedMethodException;
import org.nanoboot.powerframework.random.RandomException;
import org.nanoboot.powerframework.random.generators.linearcongruent.combined.w5.W5RandomGenerator;
import org.nanoboot.powerframework.text.CharacterType;
import org.junit.Before;
import org.junit.Test;

import static org.junit.Assert.*;
/**
 * 
 *
 * @author <a href="mailto:robertvokac@nanoboot.org">Robert Vokac</a>
 * @since 0.0.0
 */
public class RandomGeneratorTest {

    private RandomGenerator randomGenerator;

    @Before
    public void setUp() {
        this.randomGenerator = new RandomGenerator() {
            private int currentIndex = 0;
            @Override
            public long next() {
                long toReturn = randomNumbers[currentIndex];
                currentIndex++;
                if(currentIndex > (randomNumbers.length - 1)) {
                    currentIndex = 0;
                    for(int i = 0; i < randomNumbers.length; i++) {
                        randomNumbers[i] = RandomGenerator.getDefaultImplStatic().next();
                    }
                }
                return toReturn;
            }

            @Override
            public RandomGenerator getItself() {
                return this;
            }

            @Override
            public String getName() {
                return "testGenerator";
            }
        };
    }

    @Test
    public void next() {
        //setup
        //execute
        //assert
        for (int i = 0; i < 10; i++) {
            long randomLong = randomGenerator.next();
            assertEquals(randomNumbers[i], randomLong);
        }
    }

    @Test
    public void skip() {
        randomGenerator.skip();
        randomGenerator.skip();
        randomGenerator.skip();
        assertEquals(630129986537696031l, randomGenerator.next());
    }

    @Test(expected = UnsupportedMethodException.class)
    public void current() {
        randomGenerator.current();
    }

    @Test(expected = UnsupportedMethodException.class)
    public void initialSeed() {
        randomGenerator.initialSeed();
    }
    @Test(expected = UnsupportedMethodException.class)
    public void numberOfNextCalls() {
        randomGenerator.numberOfNextCalls();
    }
    @Test
    public void getItself() {
        assertEquals(this.randomGenerator.hashCode(), randomGenerator.getItself().hashCode());
    }

    @Test
    public void getNext_interval() {
        long min = Integer.MAX_VALUE;
        long max = Integer.MIN_VALUE;
        long from = 36;
        long to = 418;
        for(int i = 0; i < 9999; i++) {
            long randomLong = randomGenerator.next(from, to);
            if(randomLong < from) {
                String msg = "Random number is " + randomLong +", but it should be at least " + from;
                throw new AssertionError(msg);
            }
            if(randomLong > to) {
                String msg = "Random number is " + randomLong +", but it should be at most " + to;
                throw new AssertionError(msg);
            }
            if(randomLong < min) {
                min = randomLong;
            }
            if(randomLong > max) {
                max = randomLong;
            }
        }

        assertEquals(from, min);
        assertEquals(to, max);

    }

    @Test(expected = RandomException.class)
    public void getNext_interval_exception() {
            randomGenerator.next(10, 5);
    }

    @Test
    public void nextInt() {
        assertEquals(randomNumbers[0] % Integer.MAX_VALUE, randomGenerator.nextInt());
    }
    @Test
    public void nextInt_interval() {
        int from = 31;
        int to = 43;
        assertEquals(from + (randomNumbers[0] % (to - from + 1)), randomGenerator.nextInt(from, to));
    }
    @Test
    public void nextBoolean() {
        int from = 0;
        int to = 1;
        boolean expected = from + (randomNumbers[0] % (to - from + 1)) == 1;
        boolean returned = randomGenerator.nextInt(from, to) == 1;
        assertEquals(expected, returned);
    }
    @Test
    public void nextChar_types_numbers() {
        for(int i = 0; i < 10000; i++) {
            char ch = randomGenerator.nextChar(CharacterType.NUMBER);
            assertTrue(Character.isDigit(ch));
        }
    }
    @Test
    public void nextChar_types_lower() {
        for(int i = 0; i < 1000; i++) {
            char ch = randomGenerator.nextChar(CharacterType.LOWER_LETTER);
            assertTrue(Character.isLowerCase(ch));
        }
    }
    @Test
    public void nextChar_types_upper() {
        for(int i = 0; i < 1000; i++) {
            char ch = randomGenerator.nextChar(CharacterType.UPPER_LETTER);
            assertTrue(Character.isUpperCase((ch)));
        }
    }
    @Test
    public void nextChar_types_otherPrintable() {
        for(int i = 0; i < 1000; i++) {
            char ch = randomGenerator.nextChar(CharacterType.OTHER_PRINTABLE);
            assertFalse(Character.isWhitespace(ch));
        }
    }

    @Test(expected = UnsupportedOperationException.class)
    public void nextChar_types_notPrintable() {
            randomGenerator.nextChar(CharacterType.OTHER_NOT_PRINTABLE);
    }

    @Test
    public void nextChar_types_null() {
        for(int i = 0; i < 1000; i++) {
            char ch = randomGenerator.nextChar(null);
            boolean digit = Character.isDigit((ch));
            boolean upper = Character.isLowerCase((ch));
            boolean lower = Character.isUpperCase((ch));
            assertTrue(digit || upper || lower);
        }
    }
    @Test
    public void nextChar() {
        for(int i = 0; i < 1000; i++) {
            char ch = randomGenerator.nextChar();
            boolean digit = Character.isDigit((ch));
            boolean upper = Character.isLowerCase((ch));
            boolean lower = Character.isUpperCase((ch));
            assertTrue(digit || upper || lower);
        }
    }
    @Test
    public void nextText() {
        String returned = randomGenerator.nextText(100);
        assertEquals(100, returned.length());

        for(char ch : returned.toCharArray()) {
            boolean digit = Character.isDigit((ch));
            boolean upper = Character.isLowerCase((ch));
            boolean lower = Character.isUpperCase((ch));
            assertTrue(digit || upper || lower);
        }
    }
    @Test
    public void nextText_types_numbers() {
        String returned = randomGenerator.nextText(100, CharacterType.NUMBER);
        assertEquals(100, returned.length());
        for(char ch : returned.toCharArray()) {
            assertTrue(Character.isDigit(ch));
        }
    }
    @Test
    public void nextText_types_lower() {
        String returned = randomGenerator.nextText(100, CharacterType.LOWER_LETTER);
        for(char ch : returned.toCharArray()) {
            assertTrue(Character.isLowerCase(ch));
        }
    }
    @Test
    public void nextText_types_upper() {
        String returned = randomGenerator.nextText(100, CharacterType.UPPER_LETTER);
        for(char ch : returned.toCharArray()) {
            assertTrue(Character.isUpperCase((ch)));
        }
    }
    @Test
    public void nextText_types_otherPrintable() {
        String returned = randomGenerator.nextText(100, CharacterType.OTHER_PRINTABLE);
        for(char ch : returned.toCharArray()) {
            assertFalse(Character.isWhitespace(ch));
        }
    }
    @Test
    public void randomElement() {
        int[] array = new int[] {5, 4, 9, 6, 7, 2, 5, 4, 3, 4};

        int from = 0;
        int to = array.length - 1;

        int expectedIndex = (int) (from + (randomNumbers[0] % (to - from + 1)));

        int expected = array[expectedIndex];
        int returned = randomGenerator.randomElement(5, 4, 9, 6, 7, 2, 5, 4, 3, 4);
        assertEquals(expected, returned);
    }
    @Test
    public void getDefaultImplStatic() {
        assertEquals(W5RandomGenerator.class, RandomGenerator.getDefaultImplStatic().getClass());

    }
    @Test
    public void getName() {
        assertEquals("testGenerator", randomGenerator.getName());
    }

    private final long[] randomNumbers = new long[]{
            989766495916971073l,

            752993031226338159l,
            208354589825897774l,
            630129986537696031l,
            122750107603623909l,
            87213201598134595l,
            732467883029815946l,
            751898065858841694l,
            579435717923219006l,
            313749987799630863l,
            884714377553716529l,

            965051149136722169l,
            421787665199259631l,
            657038602346525671l,
            644714380109667537l,
            922906928077404901l,
            126313368451040874l,
            432338839333393149l,
            964607090671253533l,
            232964735789415443l,
            980665142167025862l,

            469480217825462263l,
            899326816885015702l,
            875906247414770612l,
            419271813568148348l,
            955628780979616049l,
            964851394856633044l,
            920402918873167315l,
            44542537047619878l,
            287326444325502099l,
            794814281028802127l,

            859177155807759723l,
            620591375117177024l,
            589187555016189839l,
            791188474180923574l,
            73454149626876152l,
            142674074411581083l,
            585073587365940420l,
            797609949662655103l,
            175124149742874202l,
            62869744120989896l,

            551110293305242845l,
            127931539523886339l,
            920703134552677363l,
            732981382349612211l,
            62732588721496394l,
            44561050006996382l,
            119194178707933536l,
            978123130429376315l,
            862942328036352295l,
            57296861210520041l,

            182897743270630415l,
            13207875902886200l,
            552533674022145482l,
            488462941904442l,
            863787304545250572l,
            413189539957215648l,
            229359591670445984l,
            804030022526807608l,
            596524207037097032l,
            934443234016356922l,

            76899063002059964l,
            658148485712861578l,
            598631015449229747l,
            393367604407414191l,
            926192250542061452l,
            674530714803093099l,
            35980063030147176l,
            30954039774891937l,
            833583092170077764l,
            587471530888374861l,

            910416218581766388l,
            89142545784819968l,
            151420602460129151l,
            271087393304205269l,
            290597398727472261l,
            27766748062900907l,
            446097133117871237l,
            191672387717785454l,
            797842155411794101l,
            69321366131136411l,

            196344047914946886l,
            226534298261372486l,
            490733667486015817l,
            773800494736883628l,
            780878109429919665l,
            515093295699804077l,
            357085444986459223l,
            851849805620623387l,
            867722507013561571l,
            358088420691515641l,

            924912656888885760l,
            791137295807713442l,
            692109859097581060l,
            595235990482238397l,
            909911716524758764l,
            724164923777388510l,
            586273377500475847l,
            686482600666828362l,
            122259762435890876l,
            566175710915777957l,

            824013986198676042l,
            245061253612931712l,
            167220601921039098l,
            434074374001348226l,
            668858174281724461l,
            453804170192441776l,
            859959205173779472l,
            853636563417799200l,
            479051589857657035l,
            557683456529024680l,

            296261642099843803l,
            165139612823205352l,
            155787048574294110l,
            143595911615173500l,
            673657014621949249l,
            906076400428015241l,
            513670209912993763l,
            967088312580862643l,
            7774131720357429l,
            630028985223713495l,

            25673252210732419l,
            144773850559041528l,
            439733456225171722l,
            858180869123858142l,
            211394276775897243l,
            408958963895371321l,
            423472822369839857l,
            232250260713347003l,
            180706716595929038l,
            98596496887866469l,

            875678307360992371l,
            437677508049887441l,
            125170452011422203l,
            538873642230562466l,
            245197558852455185l,
            932439585603976220l,
            855639307352288859l,
            954348029862438234l,
            414712671173750006l,
            115793895418238465l,

            729484834132583888l,
            640350420376446233l,
            818925474623589460l,
            364705040323764220l,
            967195943649962411l,
            303126878372263790l,
            360410707087981191l,
            387292627370779711l,
            700601266013880678l,
            356883926601879126l,

            22583787254966927l,
            986671573053499222l,
            998723929588817991l,
            771416126519180332l,
            474486512646652781l,
            478980122430145054l,
            207722595782621257l,
            79453748112027572l,
            527397308782140854l,
            585044216263462538l,

            125397053030499187l,
            366348072275664491l,
            77735524260232115l,
            495683911214898558l,
            589156015838720909l,
            467920113868761777l,
            162071785283760169l,
            231818370419047340l,
            64307278747146290l,
            47784451860387975l,

            980055089222892624l,
            764895420560925155l,
            507748042124615833l,
            187837301351429921l,
            161726553994583475l,
            151422489191956904l,
            995805250100891082l,
            805765551492075924l,
            779377962936816736l,
            714449863347865272l,

            204015610543644036l,
            800804416286176830l,
            646873902118250214l,
            940547022409709067l,
            355335675726436202l,
            326830247048130605l,
            343704151019838190l,
            656205611304404762l,
            877236490577055500l,
            446537683901693422l,

            451209390817813316l,
            11157320132649709l,
            140467946119304817l,
            625342666087475024l,
            342673456876050403l,
            315569879245239626l,
            207968007467613588l,
            977052991039833138l,
            277741169924035053l,
            469100868042333879l,

            210908198788613658l,
            854488919399758838l,
            3288187261221639l,
            494117930374517280l,
            731270158063398126l,
            848080613499701383l,
            358143826050226673l,
            485357819668739357l,
            994980436445591452l,
            76293103128952918l,

            778136068861694776l,
            670025736323147161l,
            352721274969489950l,
            485187894208110493l,
            702706773816545161l,
            44707306458740608l,
            944417111189439331l,
            894057190523044627l,
            30682149482009118l,
            76878633081476039l,

            553790321773052068l,
            314230667439537839l,
            177236046084452338l,
            122014325466816562l,
            161909702213081218l,
            591811608817332520l,
            217450796035684001l,
            735138186156364486l,
            681433260907839992l,
            418486890410244753l,

            680797495071076289l,
            747074640063267196l,
            970128954640424935l,
            749293492114586858l,
            11728511685421992l,
            262723921643718347l,
            52511438356285514l,
            906980307982728909l,
            714879316279668660l,
            221738484375496833l,

            623442200780086407l,
            251366198355718391l,
            369497919802436634l,
            20566205275807771l,
            122283743988898264l,
            815446371346272803l,
            316726822909350586l,
            790134832122644904l,
            501226775853693784l,
            1337249705074813l,

            453262610930759038l,
            646409148301283422l,
            115579604842016905l,
            33123012565546311l,
            978401269339561507l,
            44559377343909176l,
            19514897145295274l,
            147732651075503554l,
            62477757530514553l,
            260280421539621771l,

            324238700214102842l,
            647134533413426639l,
            667861750635862171l,
            420689767461447978l,
            772352145109448446l,
            796179608438897262l,
            775193349127889400l,
            941347947779866031l,
            584282451615184212l,
            941377827111057142l,

            96189303582019295l,
            116554265264888383l,
            64574525242522589l,
            107193428552434739l,
            245510251765329105l,
            166731170493188357l,
            809949044924133275l,
            758446436914795159l,
            703043770138182890l,
            954111788081599515l,

            136859390487133799l,
            229398128019805757l,
            651047768724581987l,
            623470636016241482l,
            442548529015475022l,
            728985363468126656l,
            601534825724343989l,
            598462429001189524l,
            14767637168520017l,
            110164501031113217l,

            2267313487106505l,
            602846630066870040l,
            942742593948916153l,
            257204231220552883l,
            41928825432205789l,
            776312071726565063l,
            684333501541382594l,
            63285132045891152l,
            522617064506888728l,
            399087258085430322l,

            369335162042873576l,
            957670343495191416l,
            894002800673663108l,
            741978997518334019l,
            128424383495722946l,
            57080626670556024l,
            485987359155298593l,
            720571698380414156l,
            316090628021588845l,
            721320013932672383l,

            616724893214386955l,
            107069018902912492l,
            335684669220671881l,
            439953298090264365l,
            668766344826342757l,
            574744557050893533l,
            837492715859226581l,
            939289420759255103l,
            98903495101419241l,
            441715606911262548l,

            711796137250653774l,
            934294998305577754l,
            197739857572958523l,
            961031575986889814l,
            330389633371653429l,
            199423454318381389l,
            992806915531521002l,
            752430128263180102l,
            762067171884748509l,
            868397567289449078l,

            978131695000691716l,
            832128538091881692l,
            332557797779603829l,
            394456793413733508l,
            180261204826305843l,
            376912355571214015l,
            699324587358673569l,
            23664544583454006l,
            327974511279267901l,
            120073261092787575l,

            563426405452397018l,
            539285614539978334l,
            785159842287844887l,
            835395581519794346l,
            304538904528755810l,
            355722924067394933l,
            872021767601702295l,
            567818208391267105l,
            58356714335016452l,
            189908048834163l,

            343501795801836652l,
            488363327236059203l,
            740530667847012558l,
            242398605490351338l,
            662829190349084673l,
            387176771976134318l,
            535572586279439811l,
            226785219604563426l,
            883971137296029526l,
            463867511805352267l,

            225406583424538674l,
            767143254638275960l,
            627578036627621188l,
            25194585179292536l,
            159276033783105316l,
            533166587231767915l,
            195503830999903877l,
            143548530694574858l,
            586403308472695038l,
            251461295483424073l,

            778235211474807028l,
            974511127821538962l,
            234351427339669293l,
            674903609403754570l,
            269821646330066486l,
            10514582197647671l,
            310058695603171887l,
            692680666938759459l,
            219702762257174799l,
            394327857010791187l,

            104120696002956785l,
            932035253132951222l,
            705303760596228724l,
            168109021472592056l,
            908473434800454594l,
            633342640675753663l,
            196262077066269791l,
            130870864471689386l,
            269052619107011939l,
            945989485807845083l,

            167108829297904344l,
            512431642627242657l,
            53724341864043852l,
            558641630923859339l,
            556959743803390749l,
            631425063603427402l,
            259979605773105171l,
            429845634788551705l,
            244769122315885960l,
            68536687575157864l,

            688477984888810823l,
            733667124080264275l,
            881307008976788145l,
            85557557143709570l,
            547241936295049024l,
            608170667337476859l,
            58957920476606510l,
            966060963172955950l,
            867800335117402924l,
            67375619265528809l,

            190969923528632347l,
            425149549991394157l,
            538334572515114711l,
            708307525017999452l,
            25983085084983815l,
            672261026943582858l,
            219667292242148547l,
            121957383179923845l,
            77600241275319820l,
            216862167739468905l,

            606810218542445629l,
            943507623528873810l,
            969116817300440475l,
            12294116326297464l,
            6701016055162488l,
            624148164468923139l,
            78300069444424190l,
            19339949012001435l,
            165626331279732152l,
            535298529150864448l,

            342297987067286266l,
            76712730862205265l,
            677273670705782858l,
            365940154344684371l,
            869554945670002296l,
            143520586808279523l,
            50570344277958922l,
            516182849234970511l,
            841581438239588733l,
            411495218013677191l,

            672042442089442583l,
            388189082933141721l,
            532413363660096075l,
            245694872166351121l,
            173457735056327844l,
            55738952212441330l,
            685288240200814731l,
            473032558397903901l,
            943476816171477314l,
            672789553202097285l,

            805925063742429491l,
            124678068431134300l,
            13921407309531118l,
            830591085998177448l,
            677885221100759302l,
            11585222315953622l,
            394914761078378599l,
            909884684182383685l,
            563688113563876970l,
            42694141238573193l,

            419578667557172439l,
            147566640180796541l,
            346803738847761056l,
            378264395597132729l,
            930271350665310527l,
            437581192162885448l,
            299930128325228815l,
            240538700114345918l,
            259199844081041850l,
            874083841019950255l,

            72260965305822261l,
            368038336038653535l,
            714122594498963544l,
            967513072020571246l,
            484984100291969771l,
            9452592082712525l,
            131894126687948613l,
            816971684689577237l,
            719491956090110547l,
            184818342461202456l,

            789755232122159266l,
            8086343656007639l,
            26699760340096952l,
            781284894742592836l,
            25114915695064791l,
            685759569040742267l,
            226989427089149728l,
            520659265356096853l,
            526051806884389203l,
            781488870876163849l,

            645865087449340773l,
            809989057949122731l,
            991235493762324907l,
            404707747203463395l,
            217109962102321421l,
            925147721894965627l,
            245797411148608332l,
            37234179444654375l,
            949620603085483027l,
            461465260728668501l,

            188139378599779749l,
            943884429997790515l,
            554924935592323568l,
            155486103337003356l,
            98429650840068725l,
            744643263103143420l,
            217347355647296224l,
            34408212331573531l,
            987662457879222179l,
            557453354831676008l,

            552194949301514231l,
            498679200317988091l,
            96173410445840903l,
            470259390124137059l,
            918846869428290795l,
            527541280515626944l,
            60266896027091098l,
            803321185229952136l,
            673869040626721885l,
            938647849874098059l,

            230922827820056946l,
            112461300772429907l,
            879634712004775205l,
            598816372145594026l,
            24781318238269431l,
            799861815797916440l,
            549153933444765585l,
            877992147843679864l,
            651571788003265307l,
            442367527434582550l,

            178372288197305720l,
            826839327598528358l,
            98774413195253460l,
            816432291769633027l,
            647962124566255034l,
            559374089603489176l,
            544605736920583258l,
            817024743564487677l,
            529396216145379263l,
            967712267561680183l,

            504376644759607435l,
            446026341255411979l,
            695828899771876248l,
            835953376807091163l,
            26855908735118584l,
            238447183576881932l,
            993827408015672199l,
            833930996349911951l,
            173076358439417285l,
            657629865761042316l,

            895867636891618365l,
            168944231763094742l,
            781190513685739063l,
            623214154683127617l,
            70075004308400680l,
            18443229148102777l,
            116932247227716414l,
            256057517353843325l,
            968929032552610408l,
            207014180855464275l,

            816936789295741802l,
            882995528260035433l,
            926023904722708169l,
            412566385816249322l,
            525718689734642016l,
            46920823614039094l,
            721803959992601933l,
            898168692225347300l,
            934485935477344407l,
            575677109964718533l,

            831409923501333949l,
            227533115045760681l,
            290666692630857576l,
            738134833093141619l,
            490488525842458863l,
            746816333196754644l,
            566021160371491180l,
            708616607887178478l,
            128659096747965617l,
            970839065344320588l,

            566062071670423991l,
            584278724241763662l,
            32552672364919489l,
            37866008535922086l,
            472450513414112684l,
            696409119018293313l,
            115075911937425774l,
            424621772752312046l,
            511569662923999567l,
            92880606000027127l,

            670098169072435269l,
            558056450072881022l,
            933378835959842396l,
            976604733923728806l,
            130322919640592354l,
            580712134609707403l,
            709037719269642214l,
            662262438038865940l,
            470453514500829867l,
            207366963580530330l,

            793214185522905948l,
            51852989050826915l,
            808587861629563903l,
            432136681955100363l,
            186653579520578302l,
            282860108445159829l,
            445767399365275146l,
            907762598905806393l,
            193073464810090860l,
            360915499408439905l,

            437807875937094684l,
            342704281471250858l,
            772853776000997108l,
            910808026239666144l,
            750773510093713546l,
            294650426417174248l,
            239395454575084931l,
            988245372995882576l,
            116788160781815856l,
            260427803867176874l,

            160765275244852731l,
            457421767272628860l,
            954387940750518382l,
            663009953334137662l,
            635210353908427372l,
            202191859706540593l,
            730592613929605495l,
            560114800359292146l,
            118680166891048312l,
            37945444124052740l,

            82561837702781039l,
            670968171275271971l,
            249762040649479013l,
            426742960251642201l,
            140131797903050209l,
            185585888977431133l,
            802883946492614673l,
            375966331892328518l,
            678515968858488398l,
            980554004646091626l,

            271282266915421027l,
            713566381108325971l,
            195484642496969703l,
            937837809451789980l,
            116587593283362818l,
            12326068884729358l,
            311248065455402864l,
            268537300065713148l,
            830395257050669915l,
            517717524945941475l,

            594706781477047517l,
            683514718104875033l,
            71811944144373759l,
            549184158793550316l,
            743962980344436770l,
            342815229646475244l,
            91820024504045781l,
            942183429383424182l,
            634567154658953208l,
            880813552161194772l,

            937450136245659108l,
            359216376088709535l,
            86868034556739793l,
            278428450887175865l,
            548644880526995838l,
            468965562244484588l,
            853471036651405927l,
            8707407169342708l,
            749597355229463749l,
            824330701508242834l,

            333814657130019606l,
            979058204952438795l,
            462673809227471775l,
            536914299306206417l,
            971285548245198442l,
            784544499996793098l,
            121899926051377836l,
            500200468001370135l,
            863713040769516555l,
            149374448127027320l,

            195968112763074517l,
            875555620594003339l,
            87922904524696016l,
            6167146666167473l,
            750019279660004038l,
            793059668144961967l,
            313591420157104300l,
            291353638706672805l,
            34005427496032037l,
            404306311566585712l,

            982104925320434595l,
            621802565491163415l,
            380546341802334942l,
            282624560308059338l,
            332765931735475715l,
            54875760860716960l,
            184820261436826411l,
            298338878388543994l,
            74076826076886381l,
            49573770474794851l,

            973395222473939314l,
            64651430740271251l,
            929010958483401561l,
            37911712122731241l,
            632987115622895492l,
            782598739454753832l,
            64661505976924920l,
            789897433985290804l,
            27464097941226613l,
            123510101443062121l,

            33498761022578628l,
            468710335459633953l,
            438430699110659170l,
            850870972391715098l,
            339812322380229534l,
            523007559257421888l,
            263416253899555937l,
            132523463308547710l,
            24838429221983195l,
            300869548000262289l,

            343418926622874467l,
            30181716442445862l,
            529364455337554114l,
            292068669069465220l,
            64769173797843942l,
            249469622904871477l,
            234469028566604888l,
            735530319785919659l,
            917242500339529288l,
            616037710995254445l,

            749129146363729104l,
            778125197706617057l,
            508374702704627684l,
            72638011840510682l,
            484654969086851340l,
            950681819081366290l,
            405552770893024111l,
            120788073807664024l,
            240225181610043759l,
            756173519263400684l,

            13886814190793329l,
            431811651361404368l,
            785644429323050469l,
            265083669668502313l,
            733468396640565119l,
            372535991695630142l,
            24818311402075021l,
            521896835131504122l,
            392125133711207535l,
            193359056782367881l,

            56285603302444520l,
            226057805358921359l,
            643195421142563932l,
            104039098798909983l,
            626861113096363266l,
            84979886179949074l,
            486561880501439802l,
            619593458713206099l,
            225357170894458053l,
            35456814333125993l,

            721058429053269174l,
            307576443080026683l,
            476143167247863238l,
            659345640009293086l,
            985001803816353351l,
            38729588081549534l,
            813458085650668800l,
            570847499633521267l,
            660108714578144950l,
            661225024935217344l,

            42748541343135302l,
            542409368779631480l,
            528124190371792375l,
            713468453464772935l,
            484088995075370541l,
            981866214702501659l,
            666616968432354481l,
            915366435818397300l,
            874789585783968537l,
            370682054020070752l,

            62400492067982047l,
            499850828608660586l,
            937308826652753434l,
            128558027742649386l,
            515758774554168845l,
            418916546792152877l,
            117458158181369773l,
            49562168386073568l,
            807699418958763493l,
            742145797937476656l,

            658882936116031548l,
            204462546453824419l,
            902445336976128875l,
            92692847289431172l,
            704173225059386342l,
            66639290013818113l,
            410809876283803885l,
            35952706968093473l,
            759178465001909493l,
            704067752087811350l,

            986846122776679407l,
            688733536926677107l,
            856981725890032261l,
            988531073663474807l,
            713944308475162539l,
            118847362753841156l,
            732154169724665416l,
            19005988695596513l,
            832967922377006760l,
            973672108375451327l,

            725363671507632935l,
            718391433167648827l,
            352442795126799491l,
            784366612685799127l,
            903475113629207005l,
            687344358628029044l,
            931153086939207082l,
            104622912997824952l,
            82734710816696367l,
            442945641213286735l,

            466668555176155455l,
            323463885452719651l,
            188346511024345667l,
            766367939017272210l,
            965557189722432799l,
            230021363393593477l,
            929823950768692802l,
            252119831032411121l,
            8663957261501470l,
            113678441668172265l,

            346067535048822494l,
            611852318957245790l,
            143506956345691962l,
            298077130751139726l,
            30362893522113816l,
            584809401672606297l,
            970507405565942077l,
            651371758660525390l,
            621096534538655650l,
            913256556326717871l,

            648137154880192988l,
            225633300626096742l,
            566328830294529543l,
            224653110464048130l,
            934793514428492485l,
            368669206081879487l,
            492047566736472873l,
            55677597207791273l,
            511090860495737174l,
            805179231181087093l,

            579465450607118895l,
            943178320920110013l,
            444370234898293122l,
            85402155273287473l,
            362296626894728546l,
            904433285064260477l,
            190330319436502780l,
            59222297912696625l,
            408990860335567709l,
            589474956992376535l,

            612081053822594602l,
            273841370377697349l,
            272219993018871562l,
            790440816539391295l,
            181577768978858005l,
            239045710732833793l,
            427305914448398239l,
            810713867022076074l,
            422004697986874870l,
            61779602162057360l,

            980850466579786024l,
            897408269714821783l,
            619814398120106912l,
            210645472288950922l,
            588031623554562447l,
            577816781653286458l,
            597096829305673645l,
            303263549224049320l,
            156355604966014545l,
            999940201019640042l,

            545864139992129854l,
            349017325128116083l,
            312747275210422873l,
            769921140030237021l,
            583681481462832740l,
            876665826353151013l,
            958730437530779530l,
            388741443686872468l,
            549591630978958697l,
            443407922528477935l,

            82305770875576464l,
            302021456870308996l,
            116477878113900295l,
            856679918251291335l,
            974226342944969019l,
            694139457609844063l,
            726774949030800224l,
            86877842882273743l,
            23661131870341092l,
            519961257582540181l,

            124870722812351703l,
            297797906052041293l,
            529300884305730318l,
            743823006938836829l,
            693356820954784852l,
            70200457779735722l,
            557950436116202181l,
            178048064580575228l,
            745694808225018793l,
            505492689124300290l,

            40688677867063716l,
            906029319339783317l,
            723220762979849441l,
            800464128881487142l,
            16347017065781833l,
            569886286729488306l,
            325425109670718805l,
            799442369608504858l,
            517422685398994930l,
            483902362458877314l,

            343944156351081038l,
            263401490344217325l,
            891805550054649417l,
            549964066230638103l,
            114442932252102499l,
            640299083036361103l,
            341716062024968086l,
            680156879554562369l,
            33197732406536887l,
            662573057092916456l,

            23166250889366276l,
            439588424130385671l,
            976147149098943410l,
            966634459881083697l,
            955059511573997874l,
            602113383584625957l,
            873746183951302433l,
            106714649853262351l,
            266085001547779077l,
            602087418558089717l,

            771162091195456127l,
            998022611394638150l,
            578861073290344770l,
            13953259172769202l,
            263059390252436849l,
            713849611657917124l,
            278753817441368442l,
            209565289634712522l,
            86628925891496630l,
            498675700983612899l,

            352975355046283550l,
            807496683264729873l,
            757414973431593798l,
            624901866090760637l,
            431198654617222493l,
            223908778956608211l,
            992095795763587040l,
            371430120032038210l,
            895172486004081377l,
            512169660024349480l,

            767660901924081084l,
            7567310010434317l,
            493691704503907170l,
            651819017168740877l,
            419256610638853952l,
            476464930284074251l,
            250867234483595881l,
            96309592020475526l,
            167816615939027947l,
            964084605323017643l,

            78696641868548689l,
            619677965603222936l,
            867538832840647115l,
            708534737809458263l,
            528951692596349161l,
            410702275077305522l,
            98861683128847623l,
            685470552066531037l,
            102158622587431874l,
            892742832708342987l,

            276250292240003760l,
            239000555803703746l,
            41085552660210325l,
            104036952726091280l,
            35294919829298821l,
            642288598713607316l,
            655668230261877292l,
            256750121560967030l,
            51978151194127888l,
            168586225957102212l,

            721423574237200365l,
            591127759846196563l,
            160139061303914939l,
            85049184808387624l,
            448955483195415375l,
            828466296971252038l,
            16440113349605274l,
            287422461146256705l,
            695577615454841932l
    };
}
